<?php

namespace Cenos\Master\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cenos\Master\Http\Requests\SkillcatRequest;
use Cenos\Master\Interfaces\SkillcatRepositoryInterface;
use Cenos\Master\Models\Skillcat;

/**
 * Resource controller class for skillcat.
 */
class SkillcatResourceController extends BaseController
{

    /**
     * Initialize skillcat resource controller.
     *
     * @param type SkillcatRepositoryInterface $skillcat
     *
     * @return null
     */
    public function __construct(SkillcatRepositoryInterface $skillcat)
    {
        parent::__construct();
        $this->repository = $skillcat;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cenos\Master\Repositories\Criteria\SkillcatResourceCriteria::class);
    }

    /**
     * Display a list of skillcat.
     *
     * @return Response
     */
    public function index(SkillcatRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cenos\Master\Repositories\Presenter\SkillcatPresenter::class)
                ->$function();
        }

        $skillcats = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('master::skillcat.names'))
            ->view('master::skillcat.index', true)
            ->data(compact('skillcats', 'view'))
            ->output();
    }

    /**
     * Display skillcat.
     *
     * @param Request $request
     * @param Model   $skillcat
     *
     * @return Response
     */
    public function show(SkillcatRequest $request, Skillcat $skillcat)
    {

        if ($skillcat->exists) {
            $view = 'master::skillcat.show';
        } else {
            $view = 'master::skillcat.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('master::skillcat.name'))
            ->data(compact('skillcat'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new skillcat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SkillcatRequest $request)
    {

        $skillcat = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::skillcat.name')) 
            ->view('master::skillcat.create', true) 
            ->data(compact('skillcat'))
            ->output();
    }

    /**
     * Create new skillcat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SkillcatRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $skillcat                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::skillcat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/skillcat/' . $skillcat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/skillcat'))
                ->redirect();
        }

    }

    /**
     * Show skillcat for editing.
     *
     * @param Request $request
     * @param Model   $skillcat
     *
     * @return Response
     */
    public function edit(SkillcatRequest $request, Skillcat $skillcat)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::skillcat.name'))
            ->view('master::skillcat.edit', true)
            ->data(compact('skillcat'))
            ->output();
    }

    /**
     * Update the skillcat.
     *
     * @param Request $request
     * @param Model   $skillcat
     *
     * @return Response
     */
    public function update(SkillcatRequest $request, Skillcat $skillcat)
    {
        try {
            $attributes = $request->all();

            $skillcat->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::skillcat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/skillcat/' . $skillcat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/skillcat/' . $skillcat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the skillcat.
     *
     * @param Model   $skillcat
     *
     * @return Response
     */
    public function destroy(SkillcatRequest $request, Skillcat $skillcat)
    {
        try {

            $skillcat->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::skillcat.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('master/skillcat/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/skillcat/' . $skillcat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple skillcat.
     *
     * @param Model   $skillcat
     *
     * @return Response
     */
    public function delete(SkillcatRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::skillcat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('master/skillcat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/skillcat'))
                ->redirect();
        }

    }

    /**
     * Restore deleted skillcats.
     *
     * @param Model   $skillcat
     *
     * @return Response
     */
    public function restore(SkillcatRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('master::skillcat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/master/skillcat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/skillcat/'))
                ->redirect();
        }

    }

}
