<?php

namespace Cenos\Master\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cenos\Master\Http\Requests\SourcecatRequest;
use Cenos\Master\Interfaces\SourcecatRepositoryInterface;
use Cenos\Master\Models\Sourcecat;

/**
 * Resource controller class for sourcecat.
 */
class SourcecatResourceController extends BaseController
{

    /**
     * Initialize sourcecat resource controller.
     *
     * @param type SourcecatRepositoryInterface $sourcecat
     *
     * @return null
     */
    public function __construct(SourcecatRepositoryInterface $sourcecat)
    {
        parent::__construct();
        $this->repository = $sourcecat;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cenos\Master\Repositories\Criteria\SourcecatResourceCriteria::class);
    }

    /**
     * Display a list of sourcecat.
     *
     * @return Response
     */
    public function index(SourcecatRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cenos\Master\Repositories\Presenter\SourcecatPresenter::class)
                ->$function();
        }

        $sourcecats = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('master::sourcecat.names'))
            ->view('master::sourcecat.index', true)
            ->data(compact('sourcecats', 'view'))
            ->output();
    }

    /**
     * Display sourcecat.
     *
     * @param Request $request
     * @param Model   $sourcecat
     *
     * @return Response
     */
    public function show(SourcecatRequest $request, Sourcecat $sourcecat)
    {

        if ($sourcecat->exists) {
            $view = 'master::sourcecat.show';
        } else {
            $view = 'master::sourcecat.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('master::sourcecat.name'))
            ->data(compact('sourcecat'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new sourcecat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SourcecatRequest $request)
    {

        $sourcecat = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::sourcecat.name')) 
            ->view('master::sourcecat.create', true) 
            ->data(compact('sourcecat'))
            ->output();
    }

    /**
     * Create new sourcecat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SourcecatRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $sourcecat                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::sourcecat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/sourcecat/' . $sourcecat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/sourcecat'))
                ->redirect();
        }

    }

    /**
     * Show sourcecat for editing.
     *
     * @param Request $request
     * @param Model   $sourcecat
     *
     * @return Response
     */
    public function edit(SourcecatRequest $request, Sourcecat $sourcecat)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::sourcecat.name'))
            ->view('master::sourcecat.edit', true)
            ->data(compact('sourcecat'))
            ->output();
    }

    /**
     * Update the sourcecat.
     *
     * @param Request $request
     * @param Model   $sourcecat
     *
     * @return Response
     */
    public function update(SourcecatRequest $request, Sourcecat $sourcecat)
    {
        try {
            $attributes = $request->all();

            $sourcecat->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::sourcecat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/sourcecat/' . $sourcecat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/sourcecat/' . $sourcecat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the sourcecat.
     *
     * @param Model   $sourcecat
     *
     * @return Response
     */
    public function destroy(SourcecatRequest $request, Sourcecat $sourcecat)
    {
        try {

            $sourcecat->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::sourcecat.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('master/sourcecat/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/sourcecat/' . $sourcecat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple sourcecat.
     *
     * @param Model   $sourcecat
     *
     * @return Response
     */
    public function delete(SourcecatRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::sourcecat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('master/sourcecat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/sourcecat'))
                ->redirect();
        }

    }

    /**
     * Restore deleted sourcecats.
     *
     * @param Model   $sourcecat
     *
     * @return Response
     */
    public function restore(SourcecatRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('master::sourcecat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/master/sourcecat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/sourcecat/'))
                ->redirect();
        }

    }

}
