<?php

namespace Cenos\Master\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cenos\Master\Http\Requests\StoreRequest;
use Cenos\Master\Interfaces\StoreRepositoryInterface;
use Cenos\Master\Models\Store;

/**
 * Resource controller class for store.
 */
class StoreResourceController extends BaseController
{

    /**
     * Initialize store resource controller.
     *
     * @param type StoreRepositoryInterface $store
     *
     * @return null
     */
    public function __construct(StoreRepositoryInterface $store)
    {
        parent::__construct();
        $this->repository = $store;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cenos\Master\Repositories\Criteria\StoreResourceCriteria::class);
    }

    /**
     * Display a list of store.
     *
     * @return Response
     */
    public function index(StoreRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cenos\Master\Repositories\Presenter\StorePresenter::class)
                ->$function();
        }

        $stores = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('master::store.names'))
            ->view('master::store.index', true)
            ->data(compact('stores', 'view'))
            ->output();
    }

    /**
     * Display store.
     *
     * @param Request $request
     * @param Model   $store
     *
     * @return Response
     */
    public function show(StoreRequest $request, Store $store)
    {

        if ($store->exists) {
            $view = 'master::store.show';
        } else {
            $view = 'master::store.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('master::store.name'))
            ->data(compact('store'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new store.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StoreRequest $request)
    {

        $store = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::store.name')) 
            ->view('master::store.create', true) 
            ->data(compact('store'))
            ->output();
    }

    /**
     * Create new store.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StoreRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $store                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::store.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/store/' . $store->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/store'))
                ->redirect();
        }

    }

    /**
     * Show store for editing.
     *
     * @param Request $request
     * @param Model   $store
     *
     * @return Response
     */
    public function edit(StoreRequest $request, Store $store)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::store.name'))
            ->view('master::store.edit', true)
            ->data(compact('store'))
            ->output();
    }

    /**
     * Update the store.
     *
     * @param Request $request
     * @param Model   $store
     *
     * @return Response
     */
    public function update(StoreRequest $request, Store $store)
    {
        try {
            $attributes = $request->all();

            $store->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::store.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/store/' . $store->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/store/' . $store->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the store.
     *
     * @param Model   $store
     *
     * @return Response
     */
    public function destroy(StoreRequest $request, Store $store)
    {
        try {

            $store->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::store.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('master/store/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/store/' . $store->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple store.
     *
     * @param Model   $store
     *
     * @return Response
     */
    public function delete(StoreRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::store.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('master/store'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/store'))
                ->redirect();
        }

    }

    /**
     * Restore deleted stores.
     *
     * @param Model   $store
     *
     * @return Response
     */
    public function restore(StoreRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('master::store.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/master/store'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/store/'))
                ->redirect();
        }

    }

}
