<?php

namespace Cenos\Master\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cenos\Master\Http\Requests\StoretypeRequest;
use Cenos\Master\Interfaces\StoretypeRepositoryInterface;
use Cenos\Master\Models\Storetype;

/**
 * Resource controller class for storetype.
 */
class StoretypeResourceController extends BaseController
{

    /**
     * Initialize storetype resource controller.
     *
     * @param type StoretypeRepositoryInterface $storetype
     *
     * @return null
     */
    public function __construct(StoretypeRepositoryInterface $storetype)
    {
        parent::__construct();
        $this->repository = $storetype;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cenos\Master\Repositories\Criteria\StoretypeResourceCriteria::class);
    }

    /**
     * Display a list of storetype.
     *
     * @return Response
     */
    public function index(StoretypeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cenos\Master\Repositories\Presenter\StoretypePresenter::class)
                ->$function();
        }

        $storetypes = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('master::storetype.names'))
            ->view('master::storetype.index', true)
            ->data(compact('storetypes', 'view'))
            ->output();
    }

    /**
     * Display storetype.
     *
     * @param Request $request
     * @param Model   $storetype
     *
     * @return Response
     */
    public function show(StoretypeRequest $request, Storetype $storetype)
    {

        if ($storetype->exists) {
            $view = 'master::storetype.show';
        } else {
            $view = 'master::storetype.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('master::storetype.name'))
            ->data(compact('storetype'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new storetype.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StoretypeRequest $request)
    {

        $storetype = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::storetype.name')) 
            ->view('master::storetype.create', true) 
            ->data(compact('storetype'))
            ->output();
    }

    /**
     * Create new storetype.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StoretypeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $storetype                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::storetype.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/storetype/' . $storetype->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/storetype'))
                ->redirect();
        }

    }

    /**
     * Show storetype for editing.
     *
     * @param Request $request
     * @param Model   $storetype
     *
     * @return Response
     */
    public function edit(StoretypeRequest $request, Storetype $storetype)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::storetype.name'))
            ->view('master::storetype.edit', true)
            ->data(compact('storetype'))
            ->output();
    }

    /**
     * Update the storetype.
     *
     * @param Request $request
     * @param Model   $storetype
     *
     * @return Response
     */
    public function update(StoretypeRequest $request, Storetype $storetype)
    {
        try {
            $attributes = $request->all();

            $storetype->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::storetype.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/storetype/' . $storetype->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/storetype/' . $storetype->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the storetype.
     *
     * @param Model   $storetype
     *
     * @return Response
     */
    public function destroy(StoretypeRequest $request, Storetype $storetype)
    {
        try {

            $storetype->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::storetype.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('master/storetype/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/storetype/' . $storetype->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple storetype.
     *
     * @param Model   $storetype
     *
     * @return Response
     */
    public function delete(StoretypeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::storetype.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('master/storetype'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/storetype'))
                ->redirect();
        }

    }

    /**
     * Restore deleted storetypes.
     *
     * @param Model   $storetype
     *
     * @return Response
     */
    public function restore(StoretypeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('master::storetype.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/master/storetype'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/storetype/'))
                ->redirect();
        }

    }

}
