<?php

namespace Cenos\Master\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cenos\Master\Http\Requests\TopiccatRequest;
use Cenos\Master\Interfaces\TopiccatRepositoryInterface;
use Cenos\Master\Models\Topiccat;

/**
 * Resource controller class for topiccat.
 */
class TopiccatResourceController extends BaseController
{

    /**
     * Initialize topiccat resource controller.
     *
     * @param type TopiccatRepositoryInterface $topiccat
     *
     * @return null
     */
    public function __construct(TopiccatRepositoryInterface $topiccat)
    {
        parent::__construct();
        $this->repository = $topiccat;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cenos\Master\Repositories\Criteria\TopiccatResourceCriteria::class);
    }

    /**
     * Display a list of topiccat.
     *
     * @return Response
     */
    public function index(TopiccatRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cenos\Master\Repositories\Presenter\TopiccatPresenter::class)
                ->$function();
        }

        $topiccats = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('master::topiccat.names'))
            ->view('master::topiccat.index', true)
            ->data(compact('topiccats', 'view'))
            ->output();
    }

    /**
     * Display topiccat.
     *
     * @param Request $request
     * @param Model   $topiccat
     *
     * @return Response
     */
    public function show(TopiccatRequest $request, Topiccat $topiccat)
    {

        if ($topiccat->exists) {
            $view = 'master::topiccat.show';
        } else {
            $view = 'master::topiccat.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('master::topiccat.name'))
            ->data(compact('topiccat'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new topiccat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TopiccatRequest $request)
    {

        $topiccat = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::topiccat.name')) 
            ->view('master::topiccat.create', true) 
            ->data(compact('topiccat'))
            ->output();
    }

    /**
     * Create new topiccat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TopiccatRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $topiccat                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::topiccat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/topiccat/' . $topiccat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/topiccat'))
                ->redirect();
        }

    }

    /**
     * Show topiccat for editing.
     *
     * @param Request $request
     * @param Model   $topiccat
     *
     * @return Response
     */
    public function edit(TopiccatRequest $request, Topiccat $topiccat)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::topiccat.name'))
            ->view('master::topiccat.edit', true)
            ->data(compact('topiccat'))
            ->output();
    }

    /**
     * Update the topiccat.
     *
     * @param Request $request
     * @param Model   $topiccat
     *
     * @return Response
     */
    public function update(TopiccatRequest $request, Topiccat $topiccat)
    {
        try {
            $attributes = $request->all();

            $topiccat->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::topiccat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('master/topiccat/' . $topiccat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/topiccat/' . $topiccat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the topiccat.
     *
     * @param Model   $topiccat
     *
     * @return Response
     */
    public function destroy(TopiccatRequest $request, Topiccat $topiccat)
    {
        try {

            $topiccat->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::topiccat.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('master/topiccat/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/topiccat/' . $topiccat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple topiccat.
     *
     * @param Model   $topiccat
     *
     * @return Response
     */
    public function delete(TopiccatRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::topiccat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('master/topiccat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/topiccat'))
                ->redirect();
        }

    }

    /**
     * Restore deleted topiccats.
     *
     * @param Model   $topiccat
     *
     * @return Response
     */
    public function restore(TopiccatRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('master::topiccat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/master/topiccat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/master/topiccat/'))
                ->redirect();
        }

    }

}
